﻿/*
 * Copyright (c) 2016-2021 STARFACE GmbH
 * Stephanienstr. 102
 * 76133 Karlsruhe
 * http://www.starface.de/
 * 
 * Author: Wolfgang Wallhäuser
 * 
 * This program is free software; you can redistribute it and/or modify
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * 
 * Requires STARFACE UCC Client for Windows v7.0.0.196 or later
 * 
 * 
 * Sample Code - Provided as is
 * 
 * This code connects to the STARFACE UCC Client for Windows.
 * When the client is in a phone call, the default multimedia sound device is muted.
 * 
 * Use case: You are listening to music using any media player software on your computer.
 * When you are in a phone call using your SoftPhone or desk phone, the music shall be muted.
 * 
 * This code connects to the STARFACE UCC Client for Windows.
 * When the client is in a phone call, the default multimedia sound device is muted.
 * You should ensure that the default multimedia sound device is not your headset for phone calls.
 * Otherwise you will not hear the other party.
 * 
 * 
 * Credits to Ray Molenkamp for providing the CoreAudio library
 * 
 */

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.ServiceModel;
using UccWrapper;
using UccWrapper.UccServiceReference;
using CoreAudio;

namespace MachLeiseWennAnruf
{
   //Form class, implements UccServerConnectionCallback interface for receiving UCC events from the STARFACE UCC Client for Windows
   public partial class Form1 : Form, UccServerConnectionCallback
   {
      //The UccClientWatcher implements some glue code for detecting a running STARFACE UCC Client for Windows
      private readonly UccWrapper.UccClientWatcher m_watcher = new UccClientWatcher(true);

      //WCF connection to the UCC Client
      private InstanceContext m_ctx = null;
      private NetTcpBinding m_binding = null;
      private EndpointAddress m_endpointAddress = null;
      private UccServerConnectionClient m_uccServerConnection = null;

      public Form1()
      {
         InitializeComponent();
      }

      private void Form1_Load(object sender, EventArgs e)
      {
         //Initially de-mute the device
         MuteMusic(false);

         //Connect to the UCC Client
         ConnectToUccClient(null);

         m_watcher.UccClientRunningStateChanged += OnUccClientRunningStateChanged;
      }

      private void Form1_FormClosing(object sender, FormClosingEventArgs e)
      {
         m_watcher.UccClientRunningStateChanged += OnUccClientRunningStateChanged;
         m_watcher.Dispose();
      }

      //When the UCC Client is launched, this event will provide the new TCP Endpoint of the running instance
      private void OnUccClientRunningStateChanged(bool isRunning, EndpointAddress tcpEndpointAddress, EndpointAddress namedPipeEndpointAddress)
      {
         if (!isRunning)
         {
            ShowConnectionState("UCC Client not running");
            MuteMusic(false);
            return;
         }

         ConnectToUccClient(tcpEndpointAddress);
      }

      private void ConnectToUccClient(EndpointAddress tcpEndpointAddress)
      {
         try
         {
            if (!m_watcher.IsUccClientRunning)
            {
               ShowConnectionState("UCC Client not running");
               return;
            }

            ShowConnectionState("UCC Client running");

            //Contect references this instance for receiving callbacks
            m_ctx = new InstanceContext(this);

            //WCF TCP Binding
            m_binding = new System.ServiceModel.NetTcpBinding(System.ServiceModel.SecurityMode.Transport);
            m_binding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Windows;
            m_endpointAddress = tcpEndpointAddress ?? m_watcher.TcpEndpointAddress;
            if (m_endpointAddress == null) return;

            //Connect to the UCC Client
            m_uccServerConnection = new UccServerConnectionClient(m_ctx, m_binding, m_endpointAddress);

            //Register for connection state and own presence events
            m_uccServerConnection.RegisterCallback((int)UccEventTypeFilter.UccOwnStateAndSettingsEvents | (int)UccEventTypeFilter.UccServerConnectionEvents);
         }
         catch (Exception)
         {
            // ignored
         }
      }

      public void OnUccEvent(UccEventBase uccEvent)
      {
         switch (uccEvent.EventType)
         {
            case UccEventType.UccServerConnectionStateEvent:
               {
                  //Server connection state changed
                  var connectionStateEvent = uccEvent as UccServerConnectionStateEvent;
                  if (connectionStateEvent == null) return;
                  if (connectionStateEvent.UciConnected) return;

                  ShowConnectionState("UCC client not connected to server");
                  MuteMusic(false);
               }
               break;

            case UccEventType.UccOwnPresenceEvent:
               {
                  //Own user state changed
                  var ownPresenceEvent = uccEvent as UccOwnPresenceEvent;
                  if (ownPresenceEvent == null) return;

                  //Set mute state depending on own telephony state 
                  switch (ownPresenceEvent.OwnTelephonyState)
                  {
                     case UccTelephonyUserState.Ringing:
                     case UccTelephonyUserState.Active:
                        ShowConnectionState("Call ringing or active");
                        MuteMusic(true);
                        break;

                     default:
                        ShowConnectionState("Not in a call");
                        MuteMusic(false);
                        break;
                  }
               }
               break;
         }
      }

      private void MuteMusic(bool mute)
      {
         try
         {
            //Find default audo endpoint and change mute state
            var enumErator = new MMDeviceEnumerator();
            var endpoint = enumErator.GetDefaultAudioEndpoint(EDataFlow.eRender, ERole.eMultimedia);
            endpoint.AudioEndpointVolume.Mute = mute;

            ShowMuteState(string.Format("Muted: {0}", mute));
         }
         catch (Exception)
         {
            // ignored
         }
      }

      private void ShowConnectionState(string connectionStateMessage)
      {
         BeginInvoke(new Action(() => { labelConnectionState.Text = connectionStateMessage; }));
      }

      private void ShowMuteState(string muteStateMessage)
      {
         BeginInvoke(new Action(() => { labelMuteState.Text = muteStateMessage; }));
      }

      private void buttonMute_Click(object sender, EventArgs e)
      {
         MuteMusic(true);
      }

      private void buttonDeMute_Click(object sender, EventArgs e)
      {
         MuteMusic(false);
      }
   }
}
